// JOGLDemo1.java

// This demo renders a smoothly shaded triangle and a flat shaded square. The
// triangle is rendered in front of the square in the default front view --
// looking toward the negative Z axis. In the back view (looking toward the
// positive Z axis), as specified by a command-line argument, the square is
// rendered in front of the triangle.

import java.awt.*;

import javax.media.opengl.*;
import javax.media.opengl.glu.*;

import javax.swing.*;

public class JOGLDemo1 extends JFrame
{
   final static int WIDTH = 300;
   final static int HEIGHT = 300;

   public JOGLDemo1 (boolean fBack)
   {
      super ("JOGLDemo #1");
      setDefaultCloseOperation (EXIT_ON_CLOSE);

      SceneRenderer sr = new SceneRenderer (fBack);
      sr.setPreferredSize (new Dimension (WIDTH, HEIGHT));

      getContentPane ().add (sr);

      pack ();
      setVisible (true);
   }

   public static void main (String [] args)
   {
      if (args.length > 1)
      {
          System.err.println ("usage: java JOGLDemo1 [back]");
          return;
      }

      final boolean fBack;                                                 
      fBack = (args.length == 1 && args [0].equalsIgnoreCase ("back"))
              ? true : false;
           
      Runnable r = new Runnable ()
                   {
                       public void run ()
                       {
                          new JOGLDemo1 (fBack);
                       }
                   };
      EventQueue.invokeLater (r);
   }
}

class SceneRenderer extends GLCanvas implements GLEventListener
{
   // The constants below identify the number of units between the viewpoint
   // and the near and far clipping planes.

   private final static double NEAR = 1.0; // Z values < NEAR are clipped
   private final static double FAR = 3.01;  // Z values > FAR are clipped

   // I've chosen 3.01 instead of 3.0 because floating-point imprecision
   // results in the triangle being clipped when the scene is viewed from the
   // back.

   private boolean fBack;

   private GLU glu = new GLU ();

   SceneRenderer (boolean fBack)
   {
      this.fBack = fBack;

      addGLEventListener (this);
   }

   public void init (GLAutoDrawable drawable)
   {
      System.out.println ("init");

      GL gl = drawable.getGL ();

      // Uncomment the following line to create a white background. The
      // default background color is black.

//      gl.glClearColor (1.0f, 1.0f, 1.0f, 1.0f);

      // Enable depth testing so that the square will appear behind the
      // triangle when the scene is viewed from the front.

      gl.glEnable (GL.GL_DEPTH_TEST);
   }

   public void display (GLAutoDrawable drawable)
   {
      System.out.println ("display");

      GL gl = drawable.getGL ();

      // Clear the drawing surface to the background color. Also clear the
      // depth buffer in preparation for a new round of depth testing.

      gl.glClear (GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

      // Reset the modelview matrix.

      gl.glLoadIdentity ();

      // The camera is currently positioned at the (0, 0, 0) origin, its lens
      // is pointing along the negative Z axis (0, 0, -1) into the screen, and
      // its orientation up-vector is (0, 1, 0). The following call positions
      // the camera at (0, 0, 2) when looking at the front of the scene, or
      // (0, 0, -2) when looking at the back of the scene. It also points the
      // lens towards the origin, and keeps the same up-vector.

      if (fBack)
          glu.gluLookAt (0.0, 0.0, -2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
      else
          glu.gluLookAt (0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

      // Make sure that smooth shading is enabled. This type of shading is
      // the default the first time display() is invoked. Because flat shading
      // is enabled later on, it becomes the new default. Without the
      // following method call, the triangle would also be flat shaded on
      // subsequent display() method invocations.

      gl.glShadeModel (GL.GL_SMOOTH);

      // Render a triangle one unit in front of the origin along the positive
      // Z axis. When viewed from the front, the triangle should fill the
      // viewport.

      gl.glBegin (GL.GL_TRIANGLES);
      gl.glColor3f (1.0f, 0.0f, 0.0f); // red at the top
      gl.glVertex3d (0.0, 1.0, 1.0);
      gl.glColor3f (0.0f, 1.0f, 0.0f); // green at the lower left
      gl.glVertex3d (-1.0, -1.0, 1.0);
      gl.glColor3f (0.0f, 0.0f, 1.0f); // blue at the lower right
      gl.glVertex3d (1.0, -1.0, 1.0);
      gl.glEnd ();

      // Color the next geometric object red via flat shading, in contrast to
      // the triangle's smooth shading.

      gl.glColor3f (1.0f, 0.0f, 0.0f);
      gl.glShadeModel (GL.GL_FLAT);

      // Render a square one unit behind the origin along the negative Z axis.
      // When viewed from the back, the square should occupy about 1/16th of
      // the viewport.

      gl.glBegin (GL.GL_QUADS);
      gl.glVertex3d (-0.25, 0.25, -1.0);
      gl.glVertex3d (-0.25, -0.25, -1.0);
      gl.glVertex3d (0.25, -0.25, -1.0);
      gl.glVertex3d (0.25, 0.25, -1.0);
      gl.glEnd ();
   }

   public void reshape (GLAutoDrawable drawable, int x, int y, int width,
                        int height)
   {
      System.out.println ("reshape");

      GL gl = drawable.getGL ();

      // We don't need to invoke gl.glViewport(x, y, width, height) because
      // this is done automatically by JOGL prior to invoking reshape().

      // Because the modelview matrix is currently in effect, we need to
      // switch to the projection matrix before we can establish a perspective
      // view.

      gl.glMatrixMode (GL.GL_PROJECTION);

      // Reset the projection matrix.

      gl.glLoadIdentity ();

      // Establish a perspective view in which any drawing in front of the
      // NEAR clipping plane or behind the FAR clipping plane is discarded.
      // The frustum is assigned the same aspect ratio as the viewport, to
      // prevent distortion.

      float aspect = (float) width/(float) height;
      gl.glFrustum (-aspect, aspect, -1.0f, 1.0f, NEAR, FAR);

      // From now on, we'll work with the modelview matrix.

      gl.glMatrixMode (GL.GL_MODELVIEW);
   }

   public void displayChanged (GLAutoDrawable drawable, boolean modeChanged,
                               boolean deviceChanged)
   {
      System.out.println ("displayChanged -- this message won't be output");
   }
}
