// JOGLDemo4.java

// This demo renders a torus over a Java2D-rendered image of the sky. It also
// lets you capture this scene to the file capture.tga by clicking the mouse
// button while the mouse is positioned over the window.

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import javax.media.opengl.*;
import javax.media.opengl.glu.*;

import javax.swing.*;

import com.sun.opengl.util.*;

public class JOGLDemo4 extends JFrame
{
   public JOGLDemo4 ()
   {
      super ("JOGLDemo #4");
      setDefaultCloseOperation (EXIT_ON_CLOSE);

      // Load the sky image.

      ImageIcon ii = new ImageIcon ("bg.jpg");

      // To render the sky image behind the torus, we first need to give this
      // drawable a non-zero alpha depth via GLCapabilities.

      GLCapabilities caps = new GLCapabilities ();
      caps.setAlphaBits (8);

      // Create the scene renderer, using its dimensions to size the GUI.

      SceneRenderer sr = new SceneRenderer (ii.getImage (), caps);
      sr.setPreferredSize (new Dimension (ii.getIconWidth (),
                                          ii.getIconHeight ()));

      setContentPane (sr);

      pack ();
      setResizable (false);
      setVisible (true);
   }

   public static void main (String [] args)
   {
      Runnable r = new Runnable ()
                   {
                       public void run ()
                       {
                          new JOGLDemo4 ();
                       }
                   };
      EventQueue.invokeLater (r);
   }
}

class SceneRenderer extends GLJPanel implements GLEventListener
{
   private final static double CAMERA_Z = 4.0;
   private final static double FOVY = 45.0; // field-of-view angle around Y 

   private final static double NEAR = 1.0; // Z values < NEAR are clipped
   private final static double FAR = 4.0;  // Z values > FAR are clipped

   private final static File CAPTURE_FILE = new File ("capture.tga");

   private GLU glu = new GLU ();

   private GLUT glut = new GLUT ();

   private Image image;

   private volatile boolean mouseWasClicked;

   SceneRenderer (Image image, GLCapabilities caps)
   {
      super (caps);

      this.image = image;

      addGLEventListener (this);

      // To render the sky image behind the torus, we also need to make this
      // drawable non-opaque.

      setOpaque (false);

      addMouseListener (new MouseAdapter ()
                        {
                            public void mouseClicked (MouseEvent me)
                            {
                               mouseWasClicked = true;
                               repaint ();
                            }
                        });
   }

   public void paintComponent (Graphics g)
   {
      // Render the sky image.

      g.drawImage (image, 0, 0, this);

      // Tell the drawable to invoke its display() method to render the torus.

      super.paintComponent (g);
   }

   public void init (GLAutoDrawable drawable)
   {
      System.out.println ("init");

      GL gl = drawable.getGL ();

      // Define the torus's specular (effect the torus has on reflected light)
      // and shininess (size and brightness of highlight) material properties.

      float[] mat_specular = { 1.0f, 1.0f, 1.0f, 1.0f };
      float[] mat_shininess = { 100.0f };

      gl.glMaterialfv (GL.GL_FRONT, GL.GL_SPECULAR, mat_specular, 0);
      gl.glMaterialfv (GL.GL_FRONT, GL.GL_SHININESS, mat_shininess, 0);

      // Define the light source's position -- to the right, above, and in
      // front of the (0, 0, 0) origin.

      float[] light_position = { 1.0f, 1.0f, 1.0f, 0.0f };

      gl.glLightfv (GL.GL_LIGHT0, GL.GL_POSITION, light_position, 0);

      // Give light source a yellow diffuse color.

      float [] light_diffuse = { 1.0f, 1.0f, 0.0f, 1.0f };

      gl.glLightfv (GL.GL_LIGHT0, GL.GL_DIFFUSE, light_diffuse, 0);

      // Enable lighting, and the single light source whose position and
      // diffuse color properties have been specified.

      gl.glEnable (GL.GL_LIGHTING);
      gl.glEnable (GL.GL_LIGHT0);

      // Enable depth buffering.

      gl.glEnable (GL.GL_DEPTH_TEST);
   }

   public void display (GLAutoDrawable drawable)
   {
      System.out.println ("display");

      GL gl = drawable.getGL ();

      // Clear the depth buffer in preparation for a new round of depth
      // testing. Also, clear the drawing surface to the background color
      // (which defaults to transparent black) if the color buffer doesn't
      // need to be preserved.

      int mask = GL.GL_DEPTH_BUFFER_BIT;

      if (!((GLJPanel) drawable).shouldPreserveColorBufferIfTranslucent ())
          mask |= GL.GL_COLOR_BUFFER_BIT;

      gl.glClear (mask);

      // Reset the modelview matrix.

      gl.glLoadIdentity ();

      // The camera is currently positioned at the (0, 0, 0) origin, its lens
      // is pointing along the negative Z axis (0, 0, -1) into the screen, and
      // its orientation up-vector is (0, 1, 0). The following call positions
      // the camera at (0, 0, CAMERA_Z), points the lens towards the origin,
      // and keeps the same up-vector.

      glu.gluLookAt (0.0, 0.0, CAMERA_Z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

      // Render a solid torus with a tube radius set to 0.5, a torus radius of 
      // 1.0, a tube wall composed with 50 sides, and a tube composed of 50
      // rings.

      glut.glutSolidTorus (0.5, 1.0, 50, 50);

      // Take screenshot.

      if (mouseWasClicked)
      {
          try
          {
              int [] viewport = { 0, 0, 0, 0 };
              gl.glGetIntegerv (GL.GL_VIEWPORT, viewport, 0);
              Screenshot.writeToTargaFile (CAPTURE_FILE, viewport [0],
                                           viewport [1], viewport [2],
                                           viewport [3], false);
          }
          catch (IOException ioe)
          {
              System.out.println (ioe.getMessage ());
          }

          mouseWasClicked = false;
      }
   }

   public void reshape (GLAutoDrawable drawable, int x, int y, int width,
                        int height)
   {
      System.out.println ("reshape");

      GL gl = drawable.getGL ();

      // We don't need to invoke gl.glViewport(x, y, width, height) because
      // this is done automatically by JOGL prior to invoking reshape().

      // Because the modelview matrix is currently in effect, we need to
      // switch to the projection matrix before we can establish a perspective
      // view.

      gl.glMatrixMode (GL.GL_PROJECTION);

      // Reset the projection matrix.

      gl.glLoadIdentity ();

      // Establish a perspective view with an FOVY-degree viewing angle based
      // on the drawable's width and height. Furthermore, set the near and far
      // clipping planes to NEAR and FAR, respectively. All drawing must take
      // place between these planes. The view volume is assigned the same
      // aspect ratio as the viewport, to prevent distortion.

      glu.gluPerspective (FOVY, width/(double) height, NEAR, FAR);

      // From now on, we'll work with the modelview matrix.

      gl.glMatrixMode (GL.GL_MODELVIEW);
   }

   public void displayChanged (GLAutoDrawable drawable, boolean modeChanged,
                               boolean deviceChanged)
   {
      System.out.println ("displayChanged -- this message won't be output");
   }
}
